@@ -47,20 +47,26 @@ class BertTagger:
         use_gpu: bool = True,
         save_pretrained: bool = True,
         pretrained_models_dir: str = "",
-        logger = None
+        logger = None,
+        cache_dir = ""
     ):
+        self.logger = logger
+        self.stdout = allow_standard_output
+        self.cache_dir = cache_dir
+
+        if self.cache_dir:
+            self._print(f"Using {self.cache_dir} as cache dir.")
+            if self.logger: self.logger.info(f"Using {self.cache_dir} as cache dir.")
 
         self.config = Config()
         self.test_mode = test_mode
         self.use_gpu = use_gpu
+        #self.cache_dir = cache_dir
 
         self.device_available_memory = None
         self.device_type = ""
         self.gpu_max_memory = 0.99
 
-        self.logger = logger
-
-        self.stdout = allow_standard_output
         self.autoadjust_batch_size = autoadjust_batch_size
         self.min_available_memory = min_available_memory
         self.sklearn_avg_function = sklearn_avg_function
@@ -240,21 +246,30 @@ class BertTagger:
             model_to_load = model_dir
             model_downloaded = True
 
+        model_kwargs = {"state_dict": state_dict, "num_labels": self.config.n_classes}
+
+        if self.cache_dir:
+             model_kwargs.update({"cache_dir": self.cache_dir})
+
         if not BertTagger.is_roberta_model(model_to_load):
-            model = BertForSequenceClassification.from_pretrained(model_to_load, state_dict = state_dict, num_labels = self.config.n_classes)
+            #os.environ['TRANSFORMERS_CACHE'] = self.cache_dir
+
+            model = BertForSequenceClassification.from_pretrained(model_to_load, **model_kwargs)
         else:
-            model = RobertaForSequenceClassification.from_pretrained(model_to_load, state_dict = state_dict, num_labels = self.config.n_classes)
+            model = RobertaForSequenceClassification.from_pretrained(model_to_load, **model_kwargs)
 
         if self.device_type == "gpu" and torch.cuda.is_available():
             torch.cuda.empty_cache()
             model.cuda()
 
         if self.save_pretrained and not model_downloaded:
+            # Save model without specified number of labels
+            del model_kwargs["num_labels"]
             # Save model without classes initialization
             if not BertTagger.is_roberta_model(model_to_load):
-                model_to_save = BertForSequenceClassification.from_pretrained(model_to_load, state_dict = state_dict)
+                model_to_save = BertForSequenceClassification.from_pretrained(model_to_load, **model_kwargs)
             else:
-                model_to_save = RobertaForSequenceClassification.from_pretrained(model_to_load, state_dict = state_dict)
+                model_to_save = RobertaForSequenceClassification.from_pretrained(model_to_load, **model_kwargs)
             self._print(f"Saving pretrained {self.config.bert_model} model files to {model_dir}.")
             model_to_save.save_pretrained(model_dir)
         return model
@@ -265,10 +280,15 @@ class BertTagger:
         model_dir = os.path.join(save_dir, BertTagger.normalize_name(bert_model), "model")
         if not os.path.exists(model_dir) or len(os.listdir(model_dir)) < 1:
             if logger: logger.info(f"Downloading pretrained model {bert_model}...")
+
+            model_kwargs = {"state_dict": state_dict}
+            if self.cache_dir:
+                 model_kwargs.update({"cache_dir": self.cache_dir})
+
             if not BertTagger.is_roberta_model(bert_model):
-                model = BertForSequenceClassification.from_pretrained(bert_model, state_dict = state_dict)
+                model = BertForSequenceClassification.from_pretrained(bert_model, **model_kwargs)
             else:
-                model = RobertaForSequenceClassification.from_pretrained(bert_model, state_dict = state_dict)
+                model = RobertaForSequenceClassification.from_pretrained(bert_model, **model_kwargs)
             model.save_pretrained(model_dir)
             if logger: logger.info(f"Download finished! Pretrained model saved into {model_dir}.")
         else:
@@ -280,10 +300,14 @@ class BertTagger:
         tokenizer_dir = os.path.join(save_dir, BertTagger.normalize_name(bert_model), "tokenizer")
         if not os.path.exists(tokenizer_dir) or len(os.listdir(tokenizer_dir)) < 1:
             if logger: logger.info(f"Downloading pretrained tokenizer {bert_model}...")
+            model_kwargs = {}
+            if self.cache_dir:
+                 model_kwargs.update({"cache_dir": self.cache_dir})
+
             if not BertTagger.is_roberta_model(bert_model):
-                tokenizer = BertTokenizer.from_pretrained(bert_model)
+                tokenizer = BertTokenizer.from_pretrained(bert_model, **model_kwargs)
             else:
-                tokenizer = RobertaTokenizer.from_pretrained(bert_model)
+                tokenizer = RobertaTokenizer.from_pretrained(bert_model, **model_kwargs)
             tokenizer.save_pretrained(tokenizer_dir)
             if logger: logger.info(f"Download finished! Pretrained tokenizer saved into {tokenizer_dir}.")
         else:
@@ -364,11 +388,12 @@ class BertTagger:
             # Tell pytorch not to bother with constructing the compute graph during
             # the forward pass, since this is only needed for backprop (training).
             with torch.no_grad():
-                (loss, logits) = self.model(b_input_ids,
+                outputs = self.model(b_input_ids,
                                        token_type_ids=b_segment_ids,
                                        attention_mask=b_input_mask,
                                        labels=b_labels)
 
+            loss, logits = outputs['loss'], outputs['logits']
             # Accumulate the validation loss.
             total_eval_loss+=loss.item()
 
@@ -439,11 +464,12 @@ class BertTagger:
             self.model.zero_grad()
 
             # Perform a forward pass (evaluate the model on this training batch).
-            loss, logits = self.model(b_input_ids,
+            outputs = self.model(b_input_ids,
                                       token_type_ids = b_segment_ids,
                                       attention_mask = b_input_mask,
                                       labels = b_labels)
 
+            loss, logits = outputs['loss'], outputs['logits']
             # Accumulate the training loss.
             total_train_loss += loss.item()
 
@@ -498,12 +524,16 @@ class BertTagger:
             model_to_load = model_dir
             model_downloaded = True
 
+        model_kwargs = {}
+        if self.cache_dir:
+            model_kwargs.update({"cache_dir": self.cache_dir})
+
         if not BertTagger.is_roberta_model(model_to_load):
             self._print("Using regular BERT tokenizer...")
-            tokenizer = BertTokenizer.from_pretrained(model_to_load)
+            tokenizer = BertTokenizer.from_pretrained(model_to_load, **model_kwargs)
         else:
             self._print("Using RobertaTokenizer...")
-            tokenizer = RobertaTokenizer.from_pretrained(model_to_load)
+            tokenizer = RobertaTokenizer.from_pretrained(model_to_load, **model_kwargs)
 
 
         if self.save_pretrained and not model_downloaded:
@@ -838,7 +868,8 @@ class BertTagger:
                 outputs = self.model(b_input_ids, token_type_ids=None,
                                      attention_mask=b_input_mask)
 
-            logits = outputs[0]
+            logits = outputs['logits']
+            #logits = outputs[0]
 
             # Move logits and labels to CPU
             logits = logits.detach().cpu()#.numpy()
